<?php

namespace App\Tasks\MassInsert;

use App\Models\Transaction;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class MassInsert6Task
{

    /**
     * insert with prepare
     *
     * PDO
     * Prepared statement insert verificar
     *
     * PREPARE transactions_plan (decimal, timestamp) AS
     * INSERT INTO transactions (value, date)
     * VALUES ($1, $2);
     *
     * EXECUTE transactions_plan(23.44, '2023-01-05 00:00:00'::timestamp);
     *
     *    1000 - Database\Seeders\TransactionSeeder ...... 1,341.07 ms DONE
     *   10000 - Database\Seeders\TransactionSeeder ..... 13,415.95 ms DONE
     *  100000 - Database\Seeders\TransactionSeeder ............ -  ms DONE
     * 1000000 - Database\Seeders\TransactionSeeder ............ -  ms DONE
     *
     * @param mixed $count
     * @return void
     */
    public function handle(int $count): void
    {
        $sql1 = 'PREPARE transactions_plan5 (decimal, timestamp) AS INSERT INTO transactions (value, date) VALUES ($1::decimal, $2::timestamp)';
        DB::unprepared($sql1);

        $transactions = Transaction::factory($count)->make();

        foreach ($transactions as $transaction) {
            $sql3 = vsprintf('EXECUTE transactions_plan5 (%s, \'%s\')', [
                (float)$transaction->value,
                $transaction->date->format('Y-m-d H:i:s')
            ]);

            DB::unprepared($sql3);
        }
    }
}
